import org.testng.annotations.Test;
import org.junit.jupiter.api.Assertions;

import java.util.*;
import java.util.function.BiFunction;
import java.util.function.Function;

/**
 * @author mmmm
 */
public class DictionaryTreeTests {

    @Test
    public void heightOfRootShouldBeZeroTest() {
        DictionaryTree unit = new DictionaryTree();
        Assertions.assertEquals(0, unit.height());
    }

    @Test
    public void heightOfWordShouldBeWordLengthTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word", 0);
        Assertions.assertEquals("word".length(), unit.height());
    }


    @Test
    public void wordShouldNotBeDuplicatedTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word", 0);
        unit.insert("word", 0);
        unit.insert("word", 0);
        Assertions.assertEquals(1, unit.allWords().size());
    }

    @Test
    public void removeWordTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word1", 0);
        unit.insert("word2", 0);
        unit.remove("word2");
        Assertions.assertEquals(1, unit.allWords().size());
        unit.remove("word1");
        Assertions.assertEquals(0, unit.allWords().size());
    }

    @Test
    public void containWordTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word1", 0);
        unit.insert("word2", 0);
        unit.insert("word3", 0);
        Assertions.assertEquals(true, unit.contains("word1"));
        unit.remove("word1");
        Assertions.assertEquals(false, unit.contains("word1"));
        Assertions.assertEquals(false, unit.contains("word"));
    }

    @Test
    public void predictWordTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word", 0);
        Optional<String> word = unit.predict("wo");
        Assertions.assertEquals(true, word.isPresent());
        Assertions.assertEquals("word", word.get());
        word = unit.predict("test");
        Assertions.assertEquals(false, word.isPresent());
    }


    @Test
    public void predictWordWithPopularityTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word", 0);
        unit.insert("wordPop", 5);
        Optional<String> word = unit.predict("wo");
        Assertions.assertEquals(true, word.isPresent());
        Assertions.assertEquals("wordPop", word.get());
    }


    @Test
    public void predictWordsTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word1", 5);
        unit.insert("word2", 4);
        unit.insert("word3", 3);
        unit.insert("word4", 2);
        unit.insert("word5", 1);
        List<String> predict = unit.predict("word", 3);
        Assertions.assertEquals(3, predict.size());
        Assertions.assertTrue(predict.contains("word1"));
        Assertions.assertTrue(predict.contains("word2"));
        Assertions.assertTrue(predict.contains("word3"));
        Assertions.assertFalse(predict.contains("word4"));
        Assertions.assertFalse(predict.contains("word5"));
    }

    @Test
    public void numLeavesTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word1", 0);
        unit.insert("word2", 0);
        unit.insert("test1", 0);
        unit.insert("test2", 0);

        Assertions.assertEquals(13, unit.numLeaves());
    }

    @Test
    public void maximumBranchingTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word1", 0);
        unit.insert("word2", 0);
        Assertions.assertEquals(2, unit.maximumBranching());
        unit.insert("test1", 0);
        unit.insert("test2", 0);
        unit.insert("test3", 0);
        unit.insert("test4", 0);
        unit.insert("test5", 0);
        Assertions.assertEquals(5, unit.maximumBranching());
    }

    @Test
    public void sizeTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word1", 0);
        unit.insert("word2", 0);
        unit.insert("word3", 0);
        Assertions.assertEquals(8, unit.size());
    }

    @Test
    public void longestWordTest() {
        DictionaryTree unit = new DictionaryTree();
        unit.insert("word1", 0);
        unit.insert("word2", 0);
        unit.insert("word3", 0);
        unit.insert("word3word3", 0);
        unit.insert("word3word3word3", 0);
        Assertions.assertEquals("word3word3word3", unit.longestWord());
    }

    @Test
    public void allWordsTest() {
        DictionaryTree unit = new DictionaryTree();
        List<String> allWords = new ArrayList<>();
        allWords.add("word1");
        allWords.add("word2");
        allWords.add("word3");
        allWords.add("word4");
        allWords.add("word5");
        allWords.forEach(unit::insert);
        unit.insert("word5");
        unit.insert("word5");
        Assertions.assertEquals(allWords, unit.allWords());
    }

    @Test
    public void foldTest() {
        DictionaryTree unit = new DictionaryTree();
        List<String> allWords = new ArrayList<>();
        allWords.add("w1");
        allWords.add("w2");
        allWords.add("w3");
        allWords.add("w4");
        allWords.add("w5");
        allWords.forEach(unit::insert);

        Assertions.assertEquals(String.join("-", allWords), unit.fold(new BiFunction<DictionaryTree, Collection<String>, String>() {
            @Override
            public String apply(DictionaryTree dictionaryTree, Collection<String> strings) {
                return String.join("-", dictionaryTree.allWords());
            }

            @Override
            public <V> BiFunction<DictionaryTree, Collection<String>, V> andThen(Function<? super String, ? extends V> after) {
                return null;
            }
        }));

    }


}
